package cipherlab.rfid.demo;

import cipherlab.rfid.device1800.config.DeviceResponse;
import cipherlab.rfid.device1800.config.EPCData;
import cipherlab.rfid.device1800.config.IConfigEventListener;
import cipherlab.rfid.device1800.config.InventoryData;
import cipherlab.rfid.device1800.config.RFIDMemoryBank;
import android.app.Activity;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ScrollView;

import android.widget.TextView;

public class AutoTestActivity extends Activity implements OnClickListener {

	private Button[] mBtn;
	private EditText mReceive;
	private TextView[] mText;
	private ScrollView mScrollView;
	
	private listener mylistener;
	private int SuccessCount = 0;
	private int FailCount = 0;
	private int LockCount = 0;
	private String _write;
	private boolean flag = false;

	protected static final int MESSAGE_SUCCESS = 1;
	protected static final int MESSAGE_FAIL = 2;
	protected static final int MESSAGE_TIMEOUT = 3;
	protected static final int MESSAGE_BUSY = 4;
	protected static final int MESSAGE_LOCK = 5;	
	protected static final int MESSAGE_WRITE = 6;
	protected static final int MESSAGE_Finish = 7;

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_autotest);

		AppHelper.SwitchCommandMode();

		mScrollView = (ScrollView) findViewById(R.id.SCROLLER_ID);
		
		mBtn = new Button[3];
		mBtn[0] = (Button) findViewById(R.id.btn_start);
		mBtn[0].setOnClickListener(this);
		mBtn[1] = (Button) findViewById(R.id.btn_stop);
		mBtn[1].setOnClickListener(this);
		mBtn[2] = (Button) findViewById(R.id.bn_exit);
		mBtn[2].setOnClickListener(this);

		mReceive = (EditText) findViewById(R.id.edt_epc);
		mReceive.setKeyListener(null);

		mText = new TextView[4];
		mText[0] = (TextView) findViewById(R.id.txt_success);
		mText[1] = (TextView) findViewById(R.id.txt_fail);
		mText[2] = (TextView) findViewById(R.id.txt_response);
		mText[3] = (TextView) findViewById(R.id.txt_information);

		mylistener = new listener();
		AppHelper.mConnector.addListener(mylistener);
	}

	@Override
	public void onClick(View v) {
		if (v == mBtn[0]) {
			flag = true;
			mBtn[0].setEnabled(false);
			mBtn[1].setEnabled(true);
			ClearDisplayData();
			AppHelper.Inventory();
		} else if (v == mBtn[1]) {
			flag = false;
			mBtn[1].setEnabled(false);
			mBtn[0].setEnabled(true);
		} else if (v == mBtn[2]) {
			flag = false;
			AppHelper.mConnector.Disconnect();
		}
	}

	private void ClearDisplayData() {
		mReceive.setText("");

		FailCount = 0;
		SuccessCount = 0;
		LockCount = 0;
		mText[0].setText(Integer.toString(SuccessCount));
		mText[1].setText(Integer.toString(FailCount));
		mText[2].setText(Integer.toString(LockCount));
		mText[3].setText("");
	}

	@Override
	public void onBackPressed() {
		AppHelper.mConnector.Disconnect();
		super.onBackPressed();
	}

	public class listener implements IConfigEventListener {
		public void onConnected() {

		}

		public void onDisconnected() {
			AppHelper.mConnector.removeListener(mylistener);
			AppHelper.ReleaseObject();
			finish();
		}

		public void onDeviceEvent(int value) {
			if ((value & 0xFF20) == 0x0020) {
				AppHelper.SwitchStatus = !AppHelper.SwitchStatus;
			}
		}

		@Override
		public void onDataTransmission(EPCData data) {
			ClearDisplayData();
		}

		@Override
		public void onInventory(InventoryData data, DeviceResponse response) {

			String txt_scan = mText[3].getText().toString();
			if(txt_scan.length() > 2000)
			{
				txt_scan = "";				
			}
			
			switch (response) {
			case OperationSuccess:				
				mText[0].setText(Integer.toString(SuccessCount));
				mReceive.setText(data.EPCData);				
				txt_scan += String.format("[Inventory] => %s\n", data.EPCData);				
				break;
			case OperationFinish: {
				txt_scan += String.format("[Inventory] Finish\n");
				break;
			}
			case OperationFail:
				txt_scan += String.format("[Inventory] Fail\n");
				break;
			case DeviceTimeOut: 
				txt_scan += String.format("[Inventory] Timeout\n");	
				break;			
			case DeviceBusy:
				txt_scan += String.format("[Inventory] Busy\n");				
				break;
			default:
				break;
			}			
			
			mText[3].setText(txt_scan);
			scrollToBottom();			
			
			if (response == DeviceResponse.OperationSuccess) {				
				_write = data.EPCData.substring(0,
						data.EPCData.length() - 4);
				
			}
			else if(flag == true)
			{
				switch (response)
				{
				  case OperationFinish:
				  case OperationFail:
				  case DeviceTimeOut:
				  case DeviceBusy:  
						mHandler.sendMessage(mHandler
								.obtainMessage(AutoTestActivity.MESSAGE_WRITE));
						break;
				default:
					break;
				}			
			}
		}

		@Override
		public void onReadTag(byte[] data, DeviceResponse response) {

		}
	}

	private void scrollToBottom()
	{
	    mScrollView.post(new Runnable()
	    { 
	        public void run()
	        { 
	            mScrollView.smoothScrollTo(0, mText[3].getBottom());
	        } 
	    });
	}
	
	private final Handler mHandler = new Handler() {
		@Override
		public void handleMessage(Message msg) {

			String txt_scan = mText[3].getText().toString();
			
			switch (msg.what) {
			case MESSAGE_Finish: {
				++SuccessCount;
				mText[0].setText(Integer.toString(SuccessCount));
				txt_scan += String.format("[RFIDDirectWriteTagByEPC] success\n");
				break;
			}
			case MESSAGE_FAIL: {
				++FailCount;
				mText[1].setText(Integer.toString(FailCount));
				txt_scan += String.format("[RFIDDirectWriteTagByEPC] fail\n");
				break;
			}
			case MESSAGE_TIMEOUT: {
				++FailCount;
				mText[1].setText(Integer.toString(FailCount));
				txt_scan += String.format("[RFIDDirectWriteTagByEPC] timeout\n");
				break;
			}
			case MESSAGE_BUSY: {
				++FailCount;
				mText[1].setText(Integer.toString(FailCount));
				txt_scan += String.format("[RFIDDirectWriteTagByEPC] device busy\n");
				break;
			}
			case MESSAGE_LOCK: {
				++LockCount;
				mText[2].setText(Integer.toString(LockCount));
				txt_scan += String.format("[RFIDDirectWriteTagByEPC] lock\n");
				break;
			}
			case MESSAGE_WRITE:		
				String epc = mReceive.getText().toString();
				AppHelper.WriteData(epc, RFIDMemoryBank.EPC, _write,
						mHandler);
				return;
			}

			mText[3].setText(txt_scan);
			scrollToBottom();			
			
			if (flag == true) {
				try {					
					Thread.sleep(1000);
					AppHelper.Inventory();
				} catch (InterruptedException e) {
					e.printStackTrace();
				}
			}
		}
	};
}
