package cipherlab.rfid.demo;

import java.io.UnsupportedEncodingException;
import org.apache.http.util.EncodingUtils;

import cipherlab.rfid.device1800.config.DeviceResponse;
import cipherlab.rfid.device1800.config.EPCData;
import cipherlab.rfid.device1800.config.IConfigEventListener;
import cipherlab.rfid.device1800.config.InventoryData;
import cipherlab.rfid.device1800.config.RFIDMemoryBank;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.app.Activity;
import android.view.Menu;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.CompoundButton;
import android.widget.CompoundButton.OnCheckedChangeListener;
import android.widget.EditText;
import android.widget.RadioButton;
import android.widget.Switch;
import android.widget.TextView;
import android.widget.Toast;

public class MainActivity extends Activity implements OnClickListener,
		OnCheckedChangeListener {

	protected static final int MESSAGE_SUCCESS = 1;
	protected static final int MESSAGE_FAIL = 2;
	protected static final int MESSAGE_TIMEOUT = 3;
	protected static final int MESSAGE_BUSY = 4;
	protected static final int MESSAGE_LOCK = 5;	
	protected static final int MESSAGE_WRITE = 6;
	protected static final int MESSAGE_Finish = 7;
	
	private RadioButton[] mSelect;
	private EditText[] mReceive;

	private Switch mSwitch;
	private Button mBtn;
	private Button mBtnExit;
	private TextView mType;
	private TextView mTitle;
	private int OriginLength = 0;
	private listener mylistener;
	private boolean displayflag = false;

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_main);

		AppHelper.SwitchCommandMode();

		mSelect = new RadioButton[4];
		mSelect[0] = (RadioButton) findViewById(R.id.radioButton1);
		mSelect[0].setOnClickListener(this);
		mSelect[1] = (RadioButton) findViewById(R.id.radioButton2);
		mSelect[1].setOnClickListener(this);
		mSelect[2] = (RadioButton) findViewById(R.id.radioButton3);
		mSelect[2].setOnClickListener(this);
		mSelect[3] = (RadioButton) findViewById(R.id.radioButton4);
		mSelect[3].setOnClickListener(this);
		mSelect[0].setChecked(true);

		mBtn = (Button) findViewById(R.id.button1);
		mBtn.setOnClickListener(this);

		mBtnExit = (Button) findViewById(R.id.button2);
		mBtnExit.setOnClickListener(this);

		mSwitch = (Switch) findViewById(R.id.switch1);
		mSwitch.setOnCheckedChangeListener(this);

		mReceive = new EditText[3];
		mReceive[0] = (EditText) findViewById(R.id.editText1);
		mReceive[0].setKeyListener(null);
		mReceive[1] = (EditText) findViewById(R.id.editText2);
		mReceive[1].setKeyListener(null);
		mReceive[2] = (EditText) findViewById(R.id.editText3);

		mType = (TextView) findViewById(R.id.textView4);
		mTitle = (TextView) findViewById(R.id.textView5);

		mylistener = new listener();
		AppHelper.mConnector.addListener(mylistener);
		ChangeControl();
	}

	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		return false;
	}

	@Override
	public void onClick(View v) {

		if (v == mSelect[0] || v == mSelect[1] || v == mSelect[2]
				|| v == mSelect[3]) {
			ChangeControl();
		} else if (v == mBtn) {
			if (mSelect[0].isChecked()) {
				ReadEPC();
			} else if (mSelect[1].isChecked()) {
				ReadTID();
			} else if (mSelect[2].isChecked()) {
				ReadUSER();
			} else if (mSelect[3].isChecked()) {
				WriteUSER();
			}
		} else if (v == mBtnExit) {
			AppHelper.mConnector.Disconnect();
		}
	}

	private void ClearDisplayData() {
		for (TextView txt : mReceive)
			txt.setText("");

		mType.setText("");
		OriginLength = 0;
	}

	private void ReadEPC() {
		if (mBtn.getText().toString().contains("Cancel")) {
			displayflag = false;
			AppHelper.InventoryCancel();
		} else {
			ClearDisplayData();
			displayflag = true;
			AppHelper.Inventory();

			if (mSwitch.isChecked() == false) {
				mBtn.setText("Cancel");
			}
		}
	}

	private boolean CheckEpc(String epc) {
		if (epc == null) {
			return false;
		} else if (epc.length() == 0) {
			return false;
		}

		return true;
	}

	private void ReadTID() {
		String epc = mReceive[0].getText().toString();
		if (CheckEpc(epc) == false) {
			Toast t = Toast.makeText(MainActivity.this, R.string.STR_EpcEmpty,
					Toast.LENGTH_SHORT);
			t.show();
			return;
		} 

		mReceive[1].setText("");
		AppHelper.ReadData(epc, RFIDMemoryBank.TID);

		if (mSwitch.isChecked() == false) {
			mBtn.setEnabled(false);
		}
	}

	private void ReadUSER() {
		String epc = mReceive[0].getText().toString();
		if (CheckEpc(epc) == false) {
			Toast t = Toast.makeText(MainActivity.this, R.string.STR_EpcEmpty,
					Toast.LENGTH_SHORT);
			t.show();
			return;
		} 

		mReceive[2].setText("");
		AppHelper.ReadData(epc, RFIDMemoryBank.User);

		if (mSwitch.isChecked() == false) {
			mBtn.setEnabled(false);
		}
	}

	private void WriteUSER() {
		String epc = mReceive[0].getText().toString();
		if (CheckEpc(epc) == false) {
			Toast t = Toast.makeText(MainActivity.this, R.string.STR_EpcEmpty,
					Toast.LENGTH_SHORT);
			t.show();
			return;
		} 

		String writeData = mReceive[2].getText().toString();
		if (writeData == null) {
			Toast t = Toast.makeText(MainActivity.this,
					R.string.MSG_UserbankEmpty, Toast.LENGTH_SHORT);
			t.show();
			return;
		} else if (writeData.length() % 2 == 1) {
			Toast t = Toast.makeText(MainActivity.this,
					"The binary cannot have an odd number of digits",
					Toast.LENGTH_SHORT);
			t.show();
			return;
		}

		if (OriginLength == 0) {
			Toast t = Toast.makeText(MainActivity.this,
					R.string.MSG_UserbankCheck, Toast.LENGTH_SHORT);
			t.show();
			return;
		}

		int offset = OriginLength - writeData.length();
		if (offset == OriginLength) {
			StringBuffer retvalue = new StringBuffer();
			for (int i = 0; i < offset; i++) {
				retvalue.append('0');
			}
			AppHelper.WriteData(epc, RFIDMemoryBank.User, retvalue.toString(),
					mHandler);
		} else if (offset > 0) {
			String change = PadRight(writeData, offset, '0');
			AppHelper.WriteData(epc, RFIDMemoryBank.User, change, mHandler);
		} else {
			AppHelper.WriteData(epc, RFIDMemoryBank.User, writeData, mHandler);
		}

		if (mSwitch.isChecked() == false) {
			mBtn.setEnabled(false);
		}
	}

	@Override
	public void onBackPressed() {
		AppHelper.mConnector.Disconnect();
		super.onBackPressed();
	}

	public class listener implements IConfigEventListener {
		public void onConnected() {

		}

		public void onDisconnected() {
			AppHelper.mConnector.removeListener(mylistener);
			AppHelper.ReleaseObject();
			finish();
		}

		public void onDeviceEvent(int value) {
			if ((value & 0xFF20) == 0x0020) {
				AppHelper.SwitchStatus = !AppHelper.SwitchStatus;
			}
		}

		@Override
		public void onDataTransmission(EPCData data) {

			if (mSwitch.isChecked() == true) {
				if (mSelect[0].isChecked()) {
					ClearDisplayData();
					mReceive[0].setText(data.TagData);
				} else if (mSelect[1].isChecked()) {
					if (data.Original == null)
						return;

					String temp = EncodingUtils.getString(data.Original, 0,
							data.Original.length, "ASCII");
					if (temp.contains("#@TRIGOFF\r")) {
						ReadTID();
					}
				} else if (mSelect[2].isChecked()) {
					if (data.Original == null)
						return;

					String temp = EncodingUtils.getString(data.Original, 0,
							data.Original.length, "ASCII");
					if (temp.contains("#@TRIGOFF\r")) {
						ReadUSER();
					}
				} else if (mSelect[3].isChecked()) {
					if (data.Original == null)
						return;

					String temp = EncodingUtils.getString(data.Original, 0,
							data.Original.length, "ASCII");
					if (temp.contains("#@TRIGOFF\r")) {
						WriteUSER();
					}
				}
			}
		}

		@Override
		public void onInventory(InventoryData data, DeviceResponse response) {

			switch (response) {
			case OperationSuccess:
				if (data != null && mSelect[0].isChecked()) {
					mReceive[0].setText(data.EPCData);
				}
				break;
			case OperationFinish: {
				Toast t = Toast.makeText(MainActivity.this,
						R.string.STR_Success, Toast.LENGTH_SHORT);
				t.show();
				if (mSwitch.isChecked() == false) {
					mBtn.setText("Action");
					mBtn.setEnabled(true);
				}
				break;
			}
			case OperationFail:
			case DeviceTimeOut: {
				if (displayflag == true) {
					Toast t = Toast.makeText(MainActivity.this,
							R.string.STR_Failed, Toast.LENGTH_SHORT);
					t.show();
				}
				if (mSwitch.isChecked() == false) {
					mBtn.setText("Action");
					mBtn.setEnabled(true);
				}
				break;
			}
			case DeviceBusy:
				Toast t = Toast.makeText(MainActivity.this, R.string.STR_Busy,
						Toast.LENGTH_SHORT);
				t.show();
				if (mSwitch.isChecked() == false) {
					mBtn.setText("Action");
					mBtn.setEnabled(true);
				}
				break;
			default:
				break;
			}
		}

		@Override
		public void onReadTag(byte[] data, DeviceResponse response) {

			switch (response) {
			case OperationSuccess: {
				if (mSelect[1].isChecked()) {
					if (data != null) {
						String temp;
						try {
							temp = Hex2Binary(data);
						} catch (UnsupportedEncodingException e) {
							temp = null;
							e.printStackTrace();
						}
						mReceive[1].setText(temp);
					}
				} else if (mSelect[2].isChecked()) {
					if (data != null) {
						String temp;
						try {
							temp = Hex2Binary(data);
							OriginLength = temp.length();
						} catch (UnsupportedEncodingException e) {
							temp = null;
							e.printStackTrace();
						}
						mReceive[2].setText(temp);
					}
				} else if (mSelect[3].isChecked()) {

				}
				break;
			}
			case OperationFinish: {
				Toast t = Toast.makeText(MainActivity.this,
						R.string.STR_Success, Toast.LENGTH_SHORT);
				t.show();
				if (mSwitch.isChecked() == false) {
					mBtn.setText("Action");
					mBtn.setEnabled(true);
				}
				break;
			}
			case OperationFail:
			case DeviceTimeOut: {
				Toast t = Toast.makeText(MainActivity.this,
						R.string.STR_Failed, Toast.LENGTH_SHORT);
				t.show();
				if (mSwitch.isChecked() == false) {
					mBtn.setText("Action");
					mBtn.setEnabled(true);
				}
				break;
			}
			case DeviceBusy:
				Toast t = Toast.makeText(MainActivity.this, R.string.STR_Busy,
						Toast.LENGTH_SHORT);
				t.show();
				if (mSwitch.isChecked() == false) {
					mBtn.setText("Action");
					mBtn.setEnabled(true);
				}
				break;
			default:
				break;
			}
		}
	}

	@Override
	public void onCheckedChanged(CompoundButton buttonView, boolean isChecked) {
		if (isChecked == true) {
			mBtn.setVisibility(View.INVISIBLE);
			mTitle.setVisibility(View.INVISIBLE);
		} else {
			mBtn.setVisibility(View.VISIBLE);
			mTitle.setVisibility(View.VISIBLE);
		}

		ChangeControl();
	}

	private void ChangeControl() {
		if (mSwitch.isChecked() == true) {
			if (mSelect[0].isChecked()) {
				AppHelper.ChangeDirectControl(true);
			} else if (mSelect[1].isChecked()) {
				AppHelper.ChangeDirectControl(false);
			} else if (mSelect[2].isChecked()) {
				AppHelper.ChangeDirectControl(false);
			} else if (mSelect[3].isChecked()) {
				AppHelper.ChangeDirectControl(false);
			}
		} else {
			AppHelper.ChangeDirectControl(false);
		}
	}

	private final Handler mHandler = new Handler() {
		@Override
		public void handleMessage(Message msg) {

			switch (msg.what) {
			case MESSAGE_SUCCESS: {
				Toast t = Toast.makeText(MainActivity.this,
						R.string.STR_Success, Toast.LENGTH_SHORT);
				t.show();
				if (mSwitch.isChecked() == false) {
					mBtn.setText("Action");
					mBtn.setEnabled(true);
				}
				break;
			}
			case MESSAGE_FAIL: {
				Toast t = Toast.makeText(MainActivity.this,
						R.string.STR_Failed, Toast.LENGTH_SHORT);
				t.show();
				if (mSwitch.isChecked() == false) {
					mBtn.setText("Action");
					mBtn.setEnabled(true);
				}
				break;
			}
			case MESSAGE_LOCK: {
				Toast t = Toast.makeText(MainActivity.this, "Tag had locked",
						Toast.LENGTH_SHORT);
				t.show();
				if (mSwitch.isChecked() == false) {
					mBtn.setText("Action");
					mBtn.setEnabled(true);
				}
				break;
			}
			}
		}
	};

	final static byte[] HEX_CHAR_TABLE = { (byte) '0', (byte) '1', (byte) '2',
			(byte) '3', (byte) '4', (byte) '5', (byte) '6', (byte) '7',
			(byte) '8', (byte) '9', (byte) 'a', (byte) 'b', (byte) 'c',
			(byte) 'd', (byte) 'e', (byte) 'f' };

	public static String Hex2Binary(byte[] raw)
			throws UnsupportedEncodingException {
		byte[] hex = new byte[2 * raw.length];
		int index = 0;

		for (byte b : raw) {
			int v = b & 0xFF;
			hex[index++] = HEX_CHAR_TABLE[v >>> 4];
			hex[index++] = HEX_CHAR_TABLE[v & 0xF];
		}
		return new String(hex, "ASCII");
	}

	public static String PadRight(String in, int totalWidth, char paddingChar) {
		if (in == null || in.length() == 0)
			return null;

		StringBuffer retvalue = new StringBuffer();
		for (int i = 0; i < totalWidth; i++) {
			retvalue.append(paddingChar);
		}
		return retvalue.insert(0, in).toString();
	}

}